
From: Olivier Croquette <ocroquette@free.fr>

Presently, a process without the capability CAP_SYS_NICE can not change 
its own policy, which is OK.

But it can also not decrease its RT priority (if scheduled with policy 
SCHED_RR or SCHED_FIFO), which is what this patch changes.

The rationale is the same as for the nice value: a process should be 
able to require less priority for itself. Increasing the priority is 
still not allowed.

This is for example useful if you give a multithreaded user process a RT 
priority, and the process would like to organize its internal threads 
using priorities also. Then you can give the process the highest 
priority needed N, and the process starts its threads with lower 
priorities: N-1, N-2...

The POSIX norm says that the permissions are implementation specific, so 
I think we can do that.

In a sense, it makes the permissions consistent whatever the policy is: 
with this patch, process scheduled by SCHED_FIFO, SCHED_RR and 
SCHED_OTHER can all decrease their priority.

From: Ingo Molnar <mingo@elte.hu>

cleaned up and merged to -mm.

Signed-off-by: Ingo Molnar <mingo@elte.hu>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 kernel/sched.c |   25 ++++++++++++++++++-------
 1 files changed, 18 insertions(+), 7 deletions(-)

diff -puN kernel/sched.c~sched-changing-rt-priority-without-cap_sys_nice kernel/sched.c
--- 25/kernel/sched.c~sched-changing-rt-priority-without-cap_sys_nice	2005-06-18 02:55:39.000000000 -0700
+++ 25-akpm/kernel/sched.c	2005-06-18 02:55:39.000000000 -0700
@@ -3532,13 +3532,24 @@ recheck:
 	if ((policy == SCHED_NORMAL) != (param->sched_priority == 0))
 		return -EINVAL;
 
-	if ((policy == SCHED_FIFO || policy == SCHED_RR) &&
-	    param->sched_priority > p->signal->rlim[RLIMIT_RTPRIO].rlim_cur &&
-	    !capable(CAP_SYS_NICE))
-		return -EPERM;
-	if ((current->euid != p->euid) && (current->euid != p->uid) &&
-	    !capable(CAP_SYS_NICE))
-		return -EPERM;
+	/*
+	 * Allow unprivileged RT tasks to decrease priority:
+	 */
+	if (!capable(CAP_SYS_NICE)) {
+		/* can't change policy */
+		if (policy != p->policy)
+			return -EPERM;
+		/* can't increase priority */
+		if (policy != SCHED_NORMAL &&
+		    param->sched_priority > p->rt_priority &&
+		    param->sched_priority >
+				p->signal->rlim[RLIMIT_RTPRIO].rlim_cur)
+			return -EPERM;
+		/* can't change other user's priorities */
+		if ((current->euid != p->euid) &&
+		    (current->euid != p->uid))
+			return -EPERM;
+	}
 
 	retval = security_task_setscheduler(p, policy, param);
 	if (retval)
_
